@@ -294,22 +294,16 @@ static int vp702x_rc_query(struct dvb_usb_device *d, u32 *event, int *state)
 static int vp702x_read_mac_addr(struct dvb_usb_device *d,u8 mac[6])
 {
 	u8 i, *buf;
-	int ret;
 	struct vp702x_device_state *st = d->priv;
 
 	mutex_lock(&st->buf_mutex);
 	buf = st->buf;
-	for (i = 6; i < 12; i++) {
-		ret = vp702x_usb_in_op(d, READ_EEPROM_REQ, i, 1,
-				       &buf[i - 6], 1);
-		if (ret < 0)
-			goto err;
-	}
+	for (i = 6; i < 12; i++)
+		vp702x_usb_in_op(d, READ_EEPROM_REQ, i, 1, &buf[i - 6], 1);
 
 	memcpy(mac, buf, 6);
-err:
 	mutex_unlock(&st->buf_mutex);
-	return ret;
+	return 0;
 }
 
 static int vp702x_frontend_attach(struct dvb_usb_adapter *adap)
