@@ -2167,11 +2167,10 @@ resubmit:
 /*
  * Start the modem : init the data and start kernel thread
  */
-static int uea_boot(struct uea_softc *sc, struct usb_interface *intf)
+static int uea_boot(struct uea_softc *sc)
 {
+	int ret, size;
 	struct intr_pkt *intr;
-	int ret = -ENOMEM;
-	int size;
 
 	uea_enters(INS_TO_USBDEV(sc));
 
@@ -2196,11 +2195,6 @@ static int uea_boot(struct uea_softc *sc, struct usb_interface *intf)
 	if (UEA_CHIP_VERSION(sc) == ADI930)
 		load_XILINX_firmware(sc);
 
-	if (intf->cur_altsetting->desc.bNumEndpoints < 1) {
-		ret = -ENODEV;
-		goto err0;
-	}
-
 	intr = kmalloc(size, GFP_KERNEL);
 	if (!intr)
 		goto err0;
@@ -2212,12 +2206,13 @@ static int uea_boot(struct uea_softc *sc, struct usb_interface *intf)
 	usb_fill_int_urb(sc->urb_int, sc->usb_dev,
 			 usb_rcvintpipe(sc->usb_dev, UEA_INTR_PIPE),
 			 intr, size, uea_intr, sc,
-			 intf->cur_altsetting->endpoint[0].desc.bInterval);
+			 sc->usb_dev->actconfig->interface[0]->altsetting[0].
+			 endpoint[0].desc.bInterval);
 
 	ret = usb_submit_urb(sc->urb_int, GFP_KERNEL);
 	if (ret < 0) {
 		uea_err(INS_TO_USBDEV(sc),
-		       "urb submition failed with error %d\n", ret);
+		       "urb submission failed with error %d\n", ret);
 		goto err1;
 	}
 
@@ -2227,7 +2222,6 @@ static int uea_boot(struct uea_softc *sc, struct usb_interface *intf)
 	sc->kthread = kthread_create(uea_kthread, sc, "ueagle-atm");
 	if (IS_ERR(sc->kthread)) {
 		uea_err(INS_TO_USBDEV(sc), "failed to create thread\n");
-		ret = PTR_ERR(sc->kthread);
 		goto err2;
 	}
 
@@ -2242,7 +2236,7 @@ err1:
 	kfree(intr);
 err0:
 	uea_leaves(INS_TO_USBDEV(sc));
-	return ret;
+	return -ENOMEM;
 }
 
 /*
@@ -2528,7 +2522,7 @@ static struct attribute *attrs[] = {
 	&dev_attr_stat_firmid.attr,
 	NULL,
 };
-static struct attribute_group attr_grp = {
+static const struct attribute_group attr_grp = {
 	.attrs = attrs,
 };
 
@@ -2603,7 +2597,7 @@ static int uea_bind(struct usbatm_data *usbatm, struct usb_interface *intf,
 	if (ret < 0)
 		goto error;
 
-	ret = uea_boot(sc, intf);
+	ret = uea_boot(sc);
 	if (ret < 0)
 		goto error_rm_grp;
 
